// CBlocks : block.h
//
// Data structures and functions for tiles
//
// See userguide.pdf for copyright information

#ifndef BLOCK_H
#define BLOCK_H

#include "misc.h"

#define NUM_BLOCKS 33

typedef enum{empty=0,wire,crossover,delta,inverter,and_gate,or_gate,nand_gate,
             nor_gate,insulator,pusher,keypress,thruster,right_fuser,
             left_fuser,right_unfuser,left_unfuser,rotator,detector,
             right_slider,left_slider,equal,pulser,set_reset,creator,
             multiplier,adder,negator,store,dir_detector,less_than,
             greater_than,toggle} blocktype;

typedef enum{north=0,east,south,west,no_direction} direction;

/* Structure containing attributes for a single block */
struct block
{
  /* Kind of block. */
  blocktype type;

  /* Direction the block is facing. */
  direction orientation;

  /* Some additional state property that some blocks may possess
   * For keypress, this stores the key that the block responds to */
  int property;

  /* Output values for this block */
  int output[4];

  /* Directions in which this block is connected */
  Boolean connect[4];
};

typedef struct block Block;

/* Structure containing additional information about a block - necessary *
 * while the grid is being processed */
struct block_move
{
  /* Flag which is set if this block has already attempted to move but
   * has been cancelled later on */
  Boolean already_cancelled;

  /* Flag which is set to the current move number if the block has already
   * been asked if it is able to move */
  int asked_if_able;

  /* Flag which is set if the block is going to move.
   * If a block is empty, this flag stores whether the block is able to be
   * moved into */
  Boolean move_requested;

  /* Direction to move in */
  direction dir;
  int xo,yo;

  /* Co-ordinates of the block which originally requested this block
   * to move.
   * If a block is empty, these are the co-ordinates of the block which is
   * about to move in, and which must be halted if another block also wants
   * to move into this one */

  int requester_x,requester_y;

  /* This flag is set if the block has already been asked to rotate */
  Boolean rotated;

  /* Flags set if the connectivity has been changed */
  Boolean connect_change[4];

  /* What the new connectivity will be after changing */
  Boolean new_connect[4];

  /* This flag is set if a new block is to be created here */
  Boolean create_block;

  /* The new block to be created at this square */
  blocktype new_block;

  /* The direction of the new block to be created */
  direction new_block_dir;
};

typedef struct block_move Block_move;

struct grid_element
{
  Block block;
  Block_move block_move;
};

typedef struct grid_element Grid_element;

/* initialise a Block */
void init_Block(Block *b);

/* initialise a Block_move */
void init_Block_move(Block_move *bm);

/* initialise a Grid_element */
void init_Grid_element(Grid_element *ge);

/* Copy one grid element to another */
void copy_Grid_element(Grid_element *source, Grid_element *destination);

#endif
