// CBlocks : edit.h
//
// Functions for editing a grid
//
// See userguide.pdf for copyright information

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "edit.h"
#include "grid.h"
#include "block.h"
#include "graphics.h"

#define OPPOSITE(x) ((direction)((x+2) % 4))
#define PREVIOUS_DIR(x) ((direction)((x+3) % 4))
#define NEXT_DIR(x) ((direction)((x+1) % 4))

extern int x_lookup[4],y_lookup[4];

/* Save the section of g marked out by x1,y1 and x2,y2 (incl.) into file */
void save_section(FILE *file, Grid *g, int x1, int y1, int x2, int y2)
{
  int xs,ys;

  /* Range check */
  if (x2>=x1 && y2>=y1 && x1>=0 && y1>=0 && x2<g->width && y2<g->height) 
  {
    /* Save width and height of section */
    fprintf(file,"%d\n%d\n",x2-x1+1,y2-y1+1);

    for (xs = x1; xs <= x2; xs++)
    {
      for (ys = y1; ys <= y2; ys++)
      {
        save_block(file,&(g->the_grid[g->active][xs][ys]->block));
      }
    }
  }
}

/* Erase the section of g marked out by x1,y1 and x2,y2 (incl.) */
void erase_section(Grid *g, int x1, int y1, int x2, int y2)
{
  int xs,ys;
  direction d;

  /* Range check */
  if (x2>=x1 && y2>=y1 && x1>=0 && y1>=0 && x2<g->width && y2<g->height) 
  {
    for (xs = x1; xs <= x2; xs++)
    {
      for (ys = y1; ys <= y2; ys++)
      {
        for (d=north; d<=west; d=(direction)(d+1))
        {
          set_connect(g,xs+x_lookup[d],ys+y_lookup[d], 
                      OPPOSITE(d), False);
        }
        init_Block(&(g->the_grid[g->active][xs][ys]->block));
      }
    }
  }
}

void disconnect_edges(Grid *g,int x1, int y1, int x2, int y2)
{
  int xs,ys;
  /* Range check */
  if (x2>=x1 && y2>=y1 && x1>=0 && y1>=0 && x2<g->width && y2<g->height) 
  {
    for (xs = x1; xs <= x2; xs++)
    {
      set_connect(g,xs,y1,north, False);
      set_connect(g,xs,y2,south, False);
    }

    for (ys = y1; ys <= y2; ys++)
    {
      set_connect(g,x1,ys,west, False);
      set_connect(g,x2,ys,east, False);
    }
  }
}

/* Iterate through the whole grid and set any inconsistent connections
 * to 'disconnected' */
void remove_inconsistent_connections(Grid *g)
{
  int x,y;

  for (x=0; x<g->width; x++)
  {
    for(y=0; y<g->height; y++)
    {
      if (x>0)
      {
        if (g->the_grid[g->active][x-1][y]->block.connect[east]
         != g->the_grid[g->active][x][y]->block.connect[west])
        {
          g->the_grid[g->active][x-1][y]->block.connect[east] = False;
          g->the_grid[g->active][x][y]->block.connect[west] = False;
        }
      }
      if (x<g->width-1)
      {
        if (g->the_grid[g->active][x+1][y]->block.connect[west]
         != g->the_grid[g->active][x][y]->block.connect[east])
        {
          g->the_grid[g->active][x+1][y]->block.connect[west] = False;
          g->the_grid[g->active][x][y]->block.connect[east] = False;
        }
      }
	  /* This was modified when the n/s direction was changed */
      if (y>0)
      {
        if (g->the_grid[g->active][x][y-1]->block.connect[north]
         != g->the_grid[g->active][x][y]->block.connect[south])
        {
          g->the_grid[g->active][x][y-1]->block.connect[north] = False;
          g->the_grid[g->active][x][y]->block.connect[south] = False;
        }
      }
      if (y<g->height-1)
      {
        if (g->the_grid[g->active][x][y+1]->block.connect[south]
         != g->the_grid[g->active][x][y]->block.connect[north])
        {
          g->the_grid[g->active][x][y+1]->block.connect[south] = False;
          g->the_grid[g->active][x][y]->block.connect[north] = False;
        }
      }
    }
  }
}

/* Load a section of a grid from file into g at coordinates x,y */
void load_section(FILE *file, Grid *g, int x, int y, int fver, int transstore)
{
  int xs,ys,width,height;
  Block tmp;

  /* Range check */
  if (x>=0 && y>=0 && x<g->width && y<g->height) 
  {
    /* Load width and height of section */
    fscanf(file,"%d\n%d\n",&width,&height);

    for (xs = x; xs < (x + width); xs++)
    {
      for (ys = y; ys < (y + height); ys++)
      {
        if (xs>=0 && ys>=0 && xs<g->width && ys<g->height) 
        {
		  block *b = &(g->the_grid[g->active][xs][ys]->block);

          load_block(file,b);
		  if (xs<g->ab_x1)
			g->ab_x1 = xs;
		  if (xs>g->ab_x2)
			g->ab_x2 = xs;
		  if (ys<g->ab_y1)
			g->ab_y1 = ys;
		  if (ys>g->ab_y2)
			g->ab_y2 = ys;

		  // version 1 of cblocks had different n,e,s,w vectors and some tiles had
		  // different orientations. This section of code converts from one to the other.
		  if (!fver)
		  {
			  // flip north and south connect and output
			  int tmp = b->output[north];
			  b->output[north] = b->output[south];
			  b->output[south] = tmp;

			  tmp = b->connect[north];
			  b->connect[north] = b->connect[south];
			  b->connect[south] = (Boolean)tmp;

			  // flip east and west
			  if (b->orientation == east || b->orientation == west)
			  {
				  b->orientation = (direction)((b->orientation + 2)%4);
			  }

			  // swap sliders, fusers and unfusers
			  if (b->type == left_slider)
				  b->type = right_slider;
			  else if (b->type == right_slider)
				  b->type = left_slider;
			  if (b->type == left_fuser)
				  b->type = right_fuser;
			  else if (b->type == right_fuser)
				  b->type = left_fuser;
			  if (b->type == left_unfuser)
				  b->type = right_unfuser;
			  else if (b->type == right_unfuser)
				  b->type = left_unfuser;

			  // change store direction
			  if (b->type == store)
			  {
				  b->orientation = (direction)((b->orientation + 2)%4);
			  }
		  
		  
			  // If this is a store block, and if we are assuming that it encodes a value
			  // to be used by a creator block, the the same transformation must be applied
			  // to the encoded value as was applied above to a tile.
			  if (b->type == store && transstore)
			  {
				  if (b->output[b->orientation] >= 4 && b->output[b->orientation] < 4*NUM_BLOCKS)
				  {
					  direction d = (direction)(b->output[b->orientation] % 4);
					  blocktype t = (blocktype)((b->output[b->orientation])/4);

					  // flip east and west
					  if (d == east || d == west)
					  {
						d = (direction)((d + 2)%4);
					  }

					  // swap sliders, fusers and unfusers
					  if (t == left_slider)
						t = right_slider;
					  else if (t == right_slider)
						t = left_slider;
					  if (t == left_fuser)
						t = right_fuser;
					  else if (t == right_fuser)
						t = left_fuser;
					  if (t == left_unfuser)
						t = right_unfuser;
					  else if (t == right_unfuser)
						t = left_unfuser;

					  // change direction for all but store
					  if (t != store)
					  {
						d = (direction)((d + 2)%4);
					  }

					  b->output[b->orientation] = t*4 + d;
				  }
			  }
		  }
        }
        else
        {
          load_block(file,&tmp);
        }
      }
    }
  }
}


/* Save a block in file */
void save_block(FILE *file, Block *b)
{
  direction d;

  fprintf(file,"%d\n%d\n",(int)(b->type),(int)(b->orientation));

  for (d = north; d<=west; d=(direction)(d+1))
  {
    fprintf(file,"%d\n",(int)(b->connect[d]));
  }

  for (d = north; d<=west; d=(direction)(d+1))
  {
    fprintf(file,"%d\n",(int)(b->output[d]));
  }

  fprintf(file,"Pr=%d\n",b->property);
}

/* Load a block from file */
void load_block(FILE *file, Block *b)
{
  direction d;
  int tmp1,tmp2;
  char c1,c2,c3;

  fscanf(file,"%d\n%d\n",&tmp1,&tmp2);

  if (tmp1>=NUM_BLOCKS)
    return;

  b->type = (blocktype)tmp1;
  b->orientation = (direction)tmp2;

  for (d = north; d<=west; d=(direction)(d+1))
  {
    fscanf(file,"%d\n",&tmp1);
    b->connect[d] = (Boolean)tmp1;
  }
  
  for (d = north; d<=west; d=(direction)(d+1))
  {
    fscanf(file,"%d\n",&tmp1);
    b->output[d] = (int)tmp1;
  }

  tmp1 = ftell(file);

  /* Look for the property delimiter - this is optional so that older
   * GRD files will still work */
  c1 = fgetc(file);
  c2 = fgetc(file);
  c3 = fgetc(file);

  if (c1=='P' && c2=='r' && c3=='=')
  {
    fscanf(file,"%d\n",&tmp1);
    b->property = tmp1;
  }
  else
  {
    fseek(file,tmp1,SEEK_SET);
  }
}

