// CBlocks : funcs.cpp
//
// Tile functions
//
// See userguide.pdf for copyright information

#include "block.h"
#include "grid.h"
#include "funcs.h"

#define BLOCK(g,a,x,y) (g->the_grid[a][x][y]->block)
#define DIR(g,a,x,y) (g->the_grid[a][x][y]->block.orientation)
#define PREVIOUS_DIR(x) ((direction)((x+3) % 4))
#define NEXT_DIR(x) ((direction)((x+1) % 4))
#define OPPOSITE(x) ((direction)((x+2) % 4))

#define MAX_SIG_VAL 65536

void (*block_funcs[NUM_BLOCKS])(Grid *g, int x, int y) =
{
 funcEmpty,
 funcWire,
 funcCrossover,
 funcDelta,
 funcInverter,
 funcAndGate,
 funcOrGate,
 funcNandGate,
 funcNorGate,
 funcInsulator,
 funcPusher,
 funcKeypress,
 funcThruster,
 funcRightFuser,
 funcLeftFuser,
 funcRightUnfuser,
 funcLeftUnfuser,
 funcRotator,
 funcDetector,
 funcRightSlider,
 funcLeftSlider,
 funcEqual,
 funcPulser,
 funcSetReset,
 funcCreator,
 funcMultiplier,
 funcAdder,
 funcNegator,
 funcStore,
 funcDirDetector,
 funcLessThan,
 funcGreaterThan,
 funcToggle,
};

extern int x_lookup[4],y_lookup[4];

extern char last_char;

void funcEmpty(Grid *g,int x, int y)
{
}

void funcWire(Grid *g,int x, int y)
{
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 
    get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));
}

void funcCrossover(Grid *g,int x, int y)
{
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 
    get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[NEXT_DIR(DIR(g,g->active,x,y))] = 
    get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
}

void funcDelta(Grid *g,int x, int y)
{
  int in = get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = in;
  
  BLOCK(g,g->inactive,x,y).output[PREVIOUS_DIR(DIR(g,g->active,x,y))] = in;
  
  BLOCK(g,g->inactive,x,y).output[NEXT_DIR(DIR(g,g->active,x,y))] = in;
}

void funcInverter(Grid *g,int x, int y)
{
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 
    !get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));
}

void funcAndGate(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = in1<in2?in1:in2;
}

void funcOrGate(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = in1>in2?in1:in2;
}

void funcNandGate(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = !(in1 && in2);
}

void funcNorGate(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = !(in1 || in2);
}

void funcInsulator(Grid *g,int x, int y)
{
}

void funcPusher(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    move_block(g,x-x_lookup[d],y-y_lookup[d],x-x_lookup[d],y-y_lookup[d],OPPOSITE(d));
  }
}

void funcKeypress(Grid *g,int x, int y)
{
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 
    g->keypress;

  g->keypress = false;
}

void funcThruster(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    move_block(g,x,y,x,y,d);
  }
}

void funcRightFuser(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    fuse_blocks(g,x-x_lookup[d],y-y_lookup[d],PREVIOUS_DIR(d),True);
  }
}

void funcLeftFuser(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    fuse_blocks(g,x-x_lookup[d],y-y_lookup[d],NEXT_DIR(d),True);
  }
}

void funcLeftUnfuser(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    fuse_blocks(g,x-x_lookup[d],y-y_lookup[d],NEXT_DIR(d),False);
  }
}

void funcRightUnfuser(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    fuse_blocks(g,x-x_lookup[d],y-y_lookup[d],PREVIOUS_DIR(d),False);
  }
}

void funcRotator(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  if (get_input(g,x,y,d))
  {
    rotate(g,x-x_lookup[d],y-y_lookup[d]);
  }
}

void funcDetector(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));

  int in = 4*get_type(g,x+x_lookup[d],y+y_lookup[d])+get_orientation(g,x+x_lookup[d],y+y_lookup[d]);
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = in;
}

void funcLeftSlider(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    move_block(g,x-x_lookup[d],y-y_lookup[d],x-x_lookup[d],y-y_lookup[d],NEXT_DIR(d));
  }
}

void funcRightSlider(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  
  if (get_input(g,x,y,d))
  {
    move_block(g,x-x_lookup[d],y-y_lookup[d],x-x_lookup[d],y-y_lookup[d],PREVIOUS_DIR(d));
  }
}

void funcEqual(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = (in1 == in2);
}

void funcPulser(Grid *g,int x, int y)
{
  int in = get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));
  
  if (BLOCK(g,g->active,x,y).property)
  {
    BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 0;
  }
  else
  {
    BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = in;
  }

  BLOCK(g,g->inactive,x,y).property = in;
}

void funcSetReset(Grid *g,int x, int y)
{
  int in2 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in1 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  if (in1 && !in2)
  {
    BLOCK(g,g->inactive,x,y).property = in1;
  }
  
  if (!in1 && in2)
  {
    BLOCK(g,g->inactive,x,y).property = 0;
  }

  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = BLOCK(g,g->active,x,y).property;
  BLOCK(g,g->inactive,x,y).output[OPPOSITE(DIR(g,g->active,x,y))] = BLOCK(g,g->active,x,y).property;
}

void funcCreator(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));
  int in = get_input(g,x,y,d);

  if (in < 4*NUM_BLOCKS && in >= 4)
  {
    create_new_block(g,x-x_lookup[d],y-y_lookup[d],(blocktype)(in/4),(direction)((d+in)%4));
  }
}

void funcMultiplier(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = (in1 * in2) % MAX_SIG_VAL;
}

void funcAdder(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = (in1 + in2) % MAX_SIG_VAL;
}

void funcNegator(Grid *g,int x, int y)
{
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 
    -get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));
}

void funcStore(Grid *g,int x, int y)
{
  int in1 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));

  int in3 = get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));

  if (in1 && in2)
  {
    BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 0;
  }

  if (in3 && BLOCK(g,g->active,x,y).output[DIR(g,g->active,x,y)]==0)
  {
    BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = in3;
  }
}

void funcDirDetector(Grid *g,int x, int y)
{
  direction d = OPPOSITE(DIR(g,g->active,x,y));

  int in = (int)get_orientation(g,x+x_lookup[d],y+y_lookup[d]);
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = in;
}

void funcLessThan(Grid *g, int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = (in1 < in2);
}

void funcGreaterThan(Grid *g, int x, int y)
{
  int in1 = get_input(g,x,y,PREVIOUS_DIR(DIR(g,g->active,x,y)));
  int in2 = get_input(g,x,y,NEXT_DIR(DIR(g,g->active,x,y)));
  
  BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = (in1 > in2);
}

void funcToggle(Grid *g,int x, int y)
{
  int in = get_input(g,x,y,OPPOSITE(DIR(g,g->active,x,y)));

  if (in && !BLOCK(g,g->active,x,y).property)
  {
    BLOCK(g,g->inactive,x,y).output[DIR(g,g->active,x,y)] = 
    !BLOCK(g,g->active,x,y).output[DIR(g,g->active,x,y)];
  }

  BLOCK(g,g->inactive,x,y).property = in;
}

