// CBlocks : graphics.h
//
// Functions for drawing tiles
// At the moment, these functions use MFC drawing functions. It should be
// easy to replace these with the functions for whichever graphics library
// you are using.
//
// See userguide.pdf for copyright information

#include "graphics.h"
#include "block.h"
#include "grid.h"
#include "misc.h"

#define SW_X(x,s) (x)
#define SW_Y(y,s) (y)
#define SE_X(x,s) ((x)+s)
#define SE_Y(y,s) (y)
#define NW_X(x,s) (x)
#define NW_Y(y,s) ((y)+s)
#define NE_X(x,s) ((x)+s)
#define NE_Y(y,s) ((y)+s)

#define CNT_X(x,s) ((x)+s/2)
#define CNT_Y(y,s) ((y)+s/2)

void (*graph_funcs[NUM_BLOCKS])(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect) =
{
 drawEmpty,
 drawWire,
 drawCrossover,
 drawDelta,
 drawInverter,
 drawAndGate,
 drawOrGate,
 drawNandGate,
 drawNorGate,
 drawInsulator,
 drawPusher,
 drawKeypress,
 drawThruster,
 drawRightFuser,
 drawLeftFuser,
 drawRightUnfuser,
 drawLeftUnfuser,
 drawRotator,
 drawDetector,
 drawRightSlider,
 drawLeftSlider,
 drawEqual,
 drawPulser,
 drawSetReset,
 drawCreator,
 drawMultiplier,
 drawAdder,
 drawNegator,
 drawStore,
 drawDirDetector,
 drawLessThan,
 drawGreaterThan,
 drawToggle,
};

extern int x_lookup[4],y_lookup[4];


/* Do translations of x and y from centre of shape */
float TranslateX(float x, float y, direction o)
{
  return x*y_lookup[o] + y*x_lookup[o];
}

float TranslateY(float x, float y, direction o)
{
  return y*y_lookup[o] - x*x_lookup[o];
}

void drawGrid(CDC *pDC,Grid *g, float xo, float yo, float w, float h,float scale)
{
  Grid_element *ge;
  int x,y;
  int x_start = ((int)((xo+TILE_SIZE*GRID_XSIZE/2)-w/2))/TILE_SIZE;
  int y_start = ((int)((yo+TILE_SIZE*GRID_YSIZE/2)-h/2))/TILE_SIZE;
  int x_end = ((int)((xo+TILE_SIZE*GRID_XSIZE/2)+w/2))/TILE_SIZE;
  int y_end = ((int)((yo+TILE_SIZE*GRID_YSIZE/2)+h/2))/TILE_SIZE;
  float pltx,plty;

  for (x=x_start; x<=x_end; x++)
  {
    for (y=y_start; y<=y_end; y++)
    {
	  pltx = ((x-GRID_XSIZE/2)*TILE_SIZE-xo)/scale;
	  plty = ((y-GRID_YSIZE/2)*TILE_SIZE-yo)/scale;

      if (x>=0 && y>=0 && x<g->width && y<g->height)
      {
        ge = g->the_grid[g->active][x][y];

        graph_funcs[ge->block.type](pDC,pltx,plty,scale,
          ge->block.orientation,ge->block.output,ge->block.connect);
      }
      else
      {
        drawFrame(pDC,pltx,plty,scale,NULL,NULL);
      }

	  if (g->isselection && x>=g->selx1 && x<=g->selx2 && y>=g->sely1 && y<=g->sely2)
	  {
		  drawSelect(pDC,pltx,plty,scale);
	  }

    }
  }
}

void drawSelect(CDC *pDC,float x, float y, float scale)
{
	CBrush cb,*oldbrush;
	float s = TILE_SIZE/scale;

	cb.CreateSolidBrush(RGB(0,0,0));

	oldbrush = pDC->SelectObject(&cb);

	pDC->SetROP2(R2_NOTXORPEN);
	pDC->Rectangle(NW_X(x,s),NW_Y(y,s),SE_X(x,s),SE_Y(y,s));
	pDC->SetROP2(R2_COPYPEN);

	pDC->SelectObject(oldbrush);
}

void drawFrame(CDC *pDC,float x, float y, float scale, int *output, Boolean *connect)
{
	CBrush cb,*oldbrush;
	CPen cp,*oldpen;
	float s = TILE_SIZE/scale;

	cb.CreateSolidBrush(RGB(255,0,0));
	cp.CreatePen(PS_SOLID,2,RGB(0,0,255));

	if (connect && connect[north])
	{
      oldpen = (CPen *)(pDC->SelectObject(&cp));
	}
	pDC->MoveTo(NW_X(x,s),NW_Y(y,s));
	pDC->LineTo(NE_X(x,s),NE_Y(y,s));
	if (connect && connect[north])
	{
      pDC->SelectObject(oldpen);
	}

	if (connect && connect[east])
	{
      oldpen = (CPen *)(pDC->SelectObject(&cp));
	}
    pDC->MoveTo(NE_X(x,s),NE_Y(y,s));
	pDC->LineTo(SE_X(x,s),SE_Y(y,s));
	if (connect && connect[east])
	{
      pDC->SelectObject(oldpen);
	}
    
	if (connect && connect[south])
	{
      oldpen = (CPen *)(pDC->SelectObject(&cp));
	}
	pDC->MoveTo(SW_X(x,s),SW_Y(y,s));
	pDC->LineTo(SE_X(x,s),SE_Y(y,s));
	if (connect && connect[south])
	{
      pDC->SelectObject(oldpen);
	}
	
	if (connect && connect[west])
	{
      oldpen = (CPen *)(pDC->SelectObject(&cp));
	}
    pDC->MoveTo(SW_X(x,s),SW_Y(y,s));
	pDC->LineTo(NW_X(x,s),NW_Y(y,s));
	if (connect && connect[west])
	{
      pDC->SelectObject(oldpen);
	}

  if (output != NULL)
  {
    if (output[north])
    {
      oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(CNT_X(x,s)-s/8,NW_Y(y,s)-s/8,
				   CNT_X(x,s)+s/8,NW_Y(y,s)+s/8);

      pDC->SelectObject(oldbrush);
	}
    if (output[east])
    {
      oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(NE_X(x,s)-s/8,CNT_Y(y,s)-s/8,
				   NE_X(x,s)+s/8,CNT_Y(y,s)+s/8);
      pDC->SelectObject(oldbrush);
    }
    if (output[south])
    {
      oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(CNT_X(x,s)-s/8,SE_Y(y,s)-s/8,
				   CNT_X(x,s)+s/8,SE_Y(y,s)+s/8);
      pDC->SelectObject(oldbrush);
    }
    if (output[west])
    {
      oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(SW_X(x,s)-s/8,CNT_Y(y,s)-s/8,
				   SW_X(x,s)+s/8,CNT_Y(y,s)+s/8);
      pDC->SelectObject(oldbrush);
    }
  }
}

void drawEmpty(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
}

void drawDelta(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw delta shafts */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));

  /* Draw delta heads */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		  cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(s/3,0,o)-s/6,cy+TranslateY(s/3,0,o)-s/6,
		  cx+TranslateX(s/3,0,o)+s/6,cy+TranslateY(s/3,0,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }

  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(-s/3,0,o)-s/6,cy+TranslateY(-s/3,0,o)-s/6,
		  cx+TranslateX(-s/3,0,o)+s/6,cy+TranslateY(-s/3,0,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawNorGate(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw nor gate shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));

  /* Draw nor gate head */
  pDC->Ellipse(
	  cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
	  cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
	  );

    /* Draw nor gate body */
  pDC->Ellipse(
	  cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	  cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	  );

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawPusher(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw pusher lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/3,o),cy+TranslateY(0,s/3,o));
  pDC->MoveTo(cx+TranslateX(-s/2,s/3,o),cy+TranslateY(-s/2,s/3,o));
  pDC->LineTo(cx+TranslateX(s/2,s/3,o),cy+TranslateY(s/2,s/3,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawRightFuser(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw fuser lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(s/2,s/2,o),cy+TranslateY(s/2,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));

  /* Draw fuser centre */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
		  cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  drawFrame(pDC,x,y,scale,output,connect);
}

void drawLeftFuser(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw fuser lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(-s/2,s/2,o),cy+TranslateY(-s/2,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));

  /* Draw fuser centre */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
		  cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  drawFrame(pDC,x,y,scale,output,connect);
}

void drawRightUnfuser(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw fuser lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(s/2,s/2,o),cy+TranslateY(s/2,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));

  /* Draw fuser centre */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(255,255,255));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
		  cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  drawFrame(pDC,x,y,scale,output,connect);
}

void drawLeftUnfuser(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw fuser lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(-s/2,s/2,o),cy+TranslateY(-s/2,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));

  /* Draw fuser centre */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(255,255,255));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
		  cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  drawFrame(pDC,x,y,scale,output,connect);
}

void drawRotator(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw rotator lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->LineTo(cx+TranslateX(-s/5,s/12,o),cy+TranslateY(-s/5,s/12,o));
  pDC->LineTo(cx+TranslateX(-s/4,s/6,o),cy+TranslateY(-s/4,s/6,o));
  pDC->LineTo(cx+TranslateX(-s/4,s/3,o),cy+TranslateY(-s/4,s/3,o));
  pDC->LineTo(cx+TranslateX(-s/5,5*s/12,o),cy+TranslateY(-s/5,5*s/12,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawRightSlider(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw slider lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,3*s/8,o),cy+TranslateY(0,3*s/8,o));
  pDC->MoveTo(cx+TranslateX(-s/2,s/4,o),cy+TranslateY(-s/2,s/4,o));
  pDC->LineTo(cx+TranslateX(s/2,s/2,o),cy+TranslateY(s/2,s/2,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawLeftSlider(CDC *pDC,float x, float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw slider lines */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,3*s/8,o),cy+TranslateY(0,3*s/8,o));
  pDC->MoveTo(cx+TranslateX(-s/2,s/2,o),cy+TranslateY(-s/2,s/2,o));
  pDC->LineTo(cx+TranslateX(s/2,s/4,o),cy+TranslateY(s/2,s/4,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawWire(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw wire shaft */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));

  /* Draw wire head */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		  cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  drawFrame(pDC,x,y,scale,output,connect);
}

void drawCrossover(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw crossover shafts */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));

  /* Draw crossover heads */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		  cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(s/3,0,o)-s/6,cy+TranslateY(s/3,0,o)-s/6,
		  cx+TranslateX(s/3,0,o)+s/6,cy+TranslateY(s/3,0,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawInverter(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw wire shaft */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));

  /* Draw wire head */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(255,255,255));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		  cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		  cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		  );
      pDC->SelectObject(oldbrush);
  }
  
  drawFrame(pDC,x,y,scale,output,connect);
}

void drawAndGate(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw and gate shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));

  /* Draw and gate body */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
	    cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	    cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	    );

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawOrGate(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw or gate shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));

    /* Draw or gate body */
  pDC->Ellipse(
	  cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	  cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	  );

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawNandGate(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw and gate shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));


  /* Draw nand gate head */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(255,255,255));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		);

      pDC->SelectObject(oldbrush);
  }

  /* Draw nand gate body */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
	    cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	    cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	    );

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawInsulator(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  drawFrame(pDC,x,y,scale,output,connect);
}

void drawKeypress(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw keypress shaft */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));

    /* Draw keypress body */
  pDC->Ellipse(
	  cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	  cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	  );

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawThruster(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw thruster shafts */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->LineTo(cx+TranslateX(s/2,s/2,o),cy+TranslateY(s/2,s/2,o));
  pDC->MoveTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->LineTo(cx+TranslateX(-s/2,s/2,o),cy+TranslateY(-s/2,s/2,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawDetector(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw detector shafts */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(s/2,-s/2,o),cy+TranslateY(s/2,-s/2,o));
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(-s/2,-s/2,o),cy+TranslateY(-s/2,-s/2,o));

  /* Draw detector body */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
	    cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	    cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	    );

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawEqual(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw equal shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));
  pDC->MoveTo(cx+TranslateX(-s/4,-s/5,o),cy+TranslateY(-s/4,-s/5,o));
  pDC->LineTo(cx+TranslateX(s/4,-s/5,o),cy+TranslateY(s/4,-s/5,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawPulser(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw pulser shaft */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));


  /* Draw pulser head */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		);

      pDC->SelectObject(oldbrush);
  }

  /* Draw pulser body */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
	    cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	    cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	    );

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawSetReset(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw setreset shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->MoveTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));
  pDC->LineTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));


  /* Draw set reset blobs */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		cx+TranslateX(-s/3,0,o)-s/6,cy+TranslateY(-s/3,0,o)-s/6,
		cx+TranslateX(-s/3,0,o)+s/6,cy+TranslateY(-s/3,0,o)+s/6
		);

      pDC->SelectObject(oldbrush);
  }

  /* Draw set reset blobs */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(255,255,255));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
	    cx+TranslateX(s/3,0,o)-s/6,cy+TranslateY(s/3,0,o)-s/6,
	    cx+TranslateX(s/3,0,o)+s/6,cy+TranslateY(s/3,0,o)+s/6
	    );

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawCreator(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw creator shafts */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(s/2,s/2,o),cy+TranslateY(s/2,s/2,o));
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(-s/2,s/2,o),cy+TranslateY(-s/2,s/2,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawMultiplier(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw multiplier shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(-s/5,0,o),cy+TranslateY(-s/5,0,o));
  pDC->MoveTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/5,0,o),cy+TranslateY(s/5,0,o));

  pDC->MoveTo(cx+TranslateX(-s/5,-s/5,o),cy+TranslateY(-s/5,-s/5,o));
  pDC->LineTo(cx+TranslateX(s/5,s/5,o),cy+TranslateY(s/5,s/5,o));
  pDC->MoveTo(cx+TranslateX(s/5,-s/5,o),cy+TranslateY(s/5,-s/5,o));
  pDC->LineTo(cx+TranslateX(-s/5,s/5,o),cy+TranslateY(-s/5,s/5,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawAdder(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw adderr shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,0,o),cy+TranslateY(0,0,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(-s/4,0,o),cy+TranslateY(-s/4,0,o));
  pDC->MoveTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/4,0,o),cy+TranslateY(s/4,0,o));

  pDC->MoveTo(cx+TranslateX(-s/5,0,o),cy+TranslateY(-s/5,0,o));
  pDC->LineTo(cx+TranslateX(s/5,0,o),cy+TranslateY(s/5,0,o));
  pDC->MoveTo(cx+TranslateX(0,-s/5,o),cy+TranslateY(0,-s/5,o));
  pDC->LineTo(cx+TranslateX(0,s/5,o),cy+TranslateY(0,s/5,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawNegator(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw negator shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->MoveTo(cx+TranslateX(s/4,0,o),cy+TranslateY(s/4,0,o));
  pDC->LineTo(cx+TranslateX(-s/4,0,o),cy+TranslateY(-s/4,0,o));


  /* Draw negator head */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		);

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawStore(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw store shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));

  /* Draw store head */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		);

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawDirDetector(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw detector shafts */
  pDC->MoveTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(s/2,-s/2,o),cy+TranslateY(s/2,-s/2,o));
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(-s/2,-s/2,o),cy+TranslateY(-s/2,-s/2,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));

  /* Draw detector body */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
	    cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	    cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	    );

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawLessThan(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw less-than shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/5,o),cy+TranslateY(0,s/5,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(-s/5,0,o),cy+TranslateY(-s/5,0,o));
  pDC->MoveTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/5,0,o),cy+TranslateY(s/5,0,o));

  pDC->MoveTo(cx+TranslateX(s/5,s/5,o),cy+TranslateY(s/5,s/5,o));
  pDC->LineTo(cx+TranslateX(-s/5,0,o),cy+TranslateY(-s/5,0,o));
  pDC->LineTo(cx+TranslateX(s/5,-s/5,o),cy+TranslateY(s/5,-s/5,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawGreaterThan(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw greater-than shafts */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,s/5,o),cy+TranslateY(0,s/5,o));
  pDC->MoveTo(cx+TranslateX(-s/2,0,o),cy+TranslateY(-s/2,0,o));
  pDC->LineTo(cx+TranslateX(-s/5,0,o),cy+TranslateY(-s/5,0,o));
  pDC->MoveTo(cx+TranslateX(s/2,0,o),cy+TranslateY(s/2,0,o));
  pDC->LineTo(cx+TranslateX(s/5,0,o),cy+TranslateY(s/5,0,o));

  pDC->MoveTo(cx+TranslateX(-s/5,s/5,o),cy+TranslateY(-s/5,s/5,o));
  pDC->LineTo(cx+TranslateX(s/5,0,o),cy+TranslateY(s/5,0,o));
  pDC->LineTo(cx+TranslateX(-s/5,-s/5,o),cy+TranslateY(-s/5,-s/5,o));

  drawFrame(pDC,x,y,scale,output,connect);
}

void drawToggle(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect)
{
  float s = TILE_SIZE/scale;
  float cx,cy;

  cx = x+s/2;
  cy = y+s/2;

  /* Draw toggle shaft */
  pDC->MoveTo(cx+TranslateX(0,s/2,o),cy+TranslateY(0,s/2,o));
  pDC->LineTo(cx+TranslateX(0,-s/2,o),cy+TranslateY(0,-s/2,o));


  /* Draw toggle head */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(255,255,255));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
		cx+TranslateX(0,s/3,o)-s/6,cy+TranslateY(0,s/3,o)-s/6,
		cx+TranslateX(0,s/3,o)+s/6,cy+TranslateY(0,s/3,o)+s/6
		);

      pDC->SelectObject(oldbrush);
  }

  /* Draw toggle body */
  {
      CBrush cb;
	  cb.CreateSolidBrush(RGB(0,0,0));
      CBrush *oldbrush = (CBrush *)(pDC->SelectObject(&cb));

	  pDC->Ellipse(
	    cx+TranslateX(0,0,o)-s/6,cy+TranslateY(0,0,o)-s/6,
	    cx+TranslateX(0,0,o)+s/6,cy+TranslateY(0,0,o)+s/6
	    );

      pDC->SelectObject(oldbrush);
  }

  drawFrame(pDC,x,y,scale,output,connect);
}
