// CBlocks : graphics.h
//
// Functions for drawing tiles
// At the moment, these functions use MFC drawing functions. It should be
// easy to replace these with the functions for whichever graphics library
// you are using.
//
// See userguide.pdf for copyright information

#ifndef GRAPHICS_H
#define GRAPHICS_H

#include "misc.h"
#include "block.h"
#include "grid.h"

/* Draw the active array of a grid starting at x_start,y_start */
void drawGrid(CDC *pDC,Grid *g, float xo, float yo, float w, float h, float scale);

/* Functions to draw the various blocks and their outputs and connectivity.
 * x,y = pixel coords to draw at. o = orientation. output = output values.
 * connect = connectivity status */

void drawSelect(CDC *pDC,float x, float y, float scale);
void drawFrame(CDC *pDC,float x, float y, float scale, int *output, Boolean *connect);
void drawEmpty(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawWire(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawCrossover(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawDelta(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawInverter(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawAndGate(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawOrGate(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawNandGate(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawNorGate(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawInsulator(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawPusher(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawKeypress(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawThruster(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawRightFuser(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawLeftFuser(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawRightUnfuser(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawLeftUnfuser(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawRotator(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawDetector(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawRightSlider(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawLeftSlider(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawEqual(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawPulser(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawSetReset(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawCreator(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawMultiplier(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawAdder(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawNegator(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawStore(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawDirDetector(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawLessThan(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawGreaterThan(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);
void drawToggle(CDC *pDC,float x,float y, float scale, direction o, int *output, Boolean *connect);

#endif
