// CBlocks : grid.h
//
// Datastructures and functions for grids
//
// See userguide.pdf for copyright information

#ifndef GRID_H
#define GRID_H

#include "misc.h"
#include "block.h"

struct grid
{
  Grid_element ***the_grid[2];

  int width,height;

  int active,inactive;

  /* Keep track of the box that bounds the activity (activity box - ab)*/
  /* TODO - implement this */
  int ab_x1,ab_y1;
  int ab_x2,ab_y2;

  bool isselection;
  int selx1, selx2, sely1, sely2;
  bool keypress;
};

typedef struct grid Grid;

/* Given an x and y coord, return the direction that they correspond to */
direction xy_to_direction(int x, int y);

/* Initialise a grid */
Grid *init_Grid(int width, int height);

/* Destroy a grid */
void destroy_Grid(Grid *g);

/* Copy the active array to the inactive array */
void copy_Grid(Grid *g);

/* Return the type of block at x,y */
blocktype get_type(Grid *g, int x, int y);

/* Return the orientation of the block at x,y */
direction get_orientation(Grid *g, int x, int y);

/* Return the direction that the block at x,y is going to move in.  Return
 * no_direction if it is not going to move */
direction get_move_dir(Grid *g, int x, int y);

/* Return the connectivity of the block at x,y in direction dir */
Boolean get_connect(Grid *g, int x, int y, direction dir);

/* Set the connectivity of the block at x,y in direction dir */
void set_connect(Grid *g, int x, int y, direction dir, Boolean connect);

/* Get the input to block x,y which comes from the block in direction dir */
int get_input(Grid *g, int x, int y, direction dir);

/* Set the output value of block x,y in direction dir */
void set_output(Grid *g, int x, int y, direction dir, int o);

/* Try to move the block at x,y in the direction dir */
void move_block(Grid *g, int x, int y, int requester_x, int requester_y, direction dir);

/* Try to fuse/unfuse the block at x,y with the block in direction dir */
void fuse_blocks(Grid *g, int x, int y, direction dir, Boolean connect);

/* Try to rotate the block at x,y */
void rotate(Grid *g, int x, int y);

/* Place a block at x,y */
void place_block(Grid *g, int x, int y, blocktype bt, direction dir);

/* Try to create a new block at x,y */
void create_new_block(Grid *g, int x, int y, blocktype bt, direction dir);

void make_selection(Grid *g, int x1, int y1, int x2, int y2);

#endif
